# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .authorized_signatory import AuthorizedSignatory

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class AuthorizedSignatoryUpdate_Set(AuthorizedSignatory):
    type: typing_extensions.Literal["set"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class AuthorizedSignatoryUpdate_Remove(pydantic.BaseModel):
    type: typing_extensions.Literal["remove"]

    class Config:
        frozen = True
        smart_union = True


AuthorizedSignatoryUpdate = typing.Union[AuthorizedSignatoryUpdate_Set, AuthorizedSignatoryUpdate_Remove]
