# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .insurance_write_off import InsuranceWriteOff
from .patient_write_off import PatientWriteOff


class WriteOff_Patient(PatientWriteOff):
    type: typing_extensions.Literal["patient"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class WriteOff_Insurance(InsuranceWriteOff):
    type: typing_extensions.Literal["insurance"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


WriteOff = typing.Union[WriteOff_Patient, WriteOff_Insurance]
