# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....commons.types.resource_page import ResourcePage
from .payer import Payer


class PayerPage(ResourcePage):
    """
    import uuid

    from candid.resources.payers.v_3 import Payer, PayerPage

    PayerPage(
        items=[
            Payer(
                payer_uuid=uuid.UUID(
                    "a6431fd2-0712-4714-b1b1-dd094daf9f42",
                ),
                payer_id="12345",
                payer_name="Payer Name",
            )
        ],
    )
    """

    items: typing.List[Payer]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}
