# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class TaskActionExecutionMethod_CloseTask(pydantic.BaseModel):
    type: typing_extensions.Literal["close_task"]

    class Config:
        frozen = True
        smart_union = True


TaskActionExecutionMethod = typing.Union[TaskActionExecutionMethod_CloseTask]
