# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .insurance_write_off_create import InsuranceWriteOffCreate
from .patient_write_off_create import PatientWriteOffCreate


class WriteOffCreate_Patient(PatientWriteOffCreate):
    type: typing_extensions.Literal["patient"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class WriteOffCreate_Insurance(InsuranceWriteOffCreate):
    type: typing_extensions.Literal["insurance"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


WriteOffCreate = typing.Union[WriteOffCreate_Patient, WriteOffCreate_Insurance]
