# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class RefundReason(str, enum.Enum):
    OVERCHARGED = "OVERCHARGED"

    def visit(self, overcharged: typing.Callable[[], T_Result]) -> T_Result:
        if self is RefundReason.OVERCHARGED:
            return overcharged()
