# This file was auto-generated by Fern from our API Definition.

from .contract_is_linked_to_fee_schedule_http_error import ContractIsLinkedToFeeScheduleHttpError

__all__ = ["ContractIsLinkedToFeeScheduleHttpError"]
