# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .....commons.types.regions import Regions

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class RegionsUpdate_Set(pydantic.BaseModel):
    type: typing_extensions.Literal["set"]
    value: Regions

    class Config:
        frozen = True
        smart_union = True


class RegionsUpdate_Remove(pydantic.BaseModel):
    type: typing_extensions.Literal["remove"]

    class Config:
        frozen = True
        smart_union = True


RegionsUpdate = typing.Union[RegionsUpdate_Set, RegionsUpdate_Remove]
