# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .rx_cui import RxCui

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Medication(pydantic.BaseModel):
    """
    from candid.resources.encounters.v_4 import Medication

    Medication(
        name="Lisinopril",
        rx_cui="860975",
        dosage="10mg",
        dosage_form="Tablet",
        frequency="Once Daily",
        as_needed=True,
    )
    """

    name: str
    rx_cui: typing.Optional[RxCui] = None
    dosage: typing.Optional[str] = None
    dosage_form: typing.Optional[str] = None
    frequency: typing.Optional[str] = None
    as_needed: typing.Optional[bool] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}
